# -*- coding: utf-8 -*-

from hashlib import md5


class Policy(object):

    kind = NotImplemented

    def __init__(self, factory, next_policy=None, service='bpolicy'):
        self.factory = factory
        self.next_policy = next_policy
        self.service = service

    def discount_quota(self, discount):
        if self.next_policy:
            self.next_policy.discount_quota(discount)

    def _gen_factory_signature(self):
        return md5(str(self.factory.__dict__)).hexdigest()[:7]

    def _gen_memcache_key(self, mc_prefix, identity):
        return '{mc_prefix}{service}:{kind}:{factory_signature}:{identity}'.format(
            mc_prefix=self.factory.mc_prefix,
            service=self.service,
            kind=self.kind,
            factory_signature=self._gen_factory_signature(),
            identity=identity)

    def check_policy(self, identity):
        if self.next_policy:
            self.next_policy.check_policy(identity)


class PolicyFactory(object):

    policy_class = NotImplemented

    def instance(self, next_policy=None):
        return self.policy_class(self, next_policy)



# -*- coding: utf-8 -*-


from .consts import POLICY_KIND
from .error import PolicyError
from .base import Policy, PolicyFactory


class RatedPolicy(Policy):

    kind = POLICY_KIND.RATED

    def __init__(self, factory, next_policy=None):
        self.quota = factory.quota
        super(RatedPolicy, self).__init__(factory, next_policy)

    def discount_quota(self, discount):
        self.quota = self.quota * discount
        super(RatedPolicy, self).discount_quota(discount)

    def check_policy(self, identity):
        mc_key = self._gen_memcache_key(self.factory.mc_prefix, identity)
        current_counter = self.factory.mc_client.get(mc_key)
        if current_counter is None:
            current_counter = 1
            self.factory.mc_client.set(mc_key, 1, self.factory.interval)
        else:
            current_counter += 1
            self.factory.mc_client.incr(mc_key, 1)
        if current_counter > self.quota:
            raise PolicyError(self, 'max quota exceed')
        super(RatedPolicy, self).check_policy(identity)


class RatedPolicyFactory(PolicyFactory):

    policy_class = RatedPolicy

    def __init__(self, interval, quota, mc_client, mc_prefix=''):
        self.interval = interval
        self.quota = quota
        self.mc_client = mc_client
        self.mc_prefix = mc_prefix



# -*- coding: utf-8 -*-

import csv
import IPy
import os.path

from .consts import POLICY_KIND
from .base import Policy, PolicyFactory


class CERNetPolicy(Policy):

    kind = POLICY_KIND.CERNET

    def __init__(self, factory, next_policy=None):
        self.discount = factory.discount
        super(CERNetPolicy, self).__init__(factory, next_policy)

    def is_cernet_ipaddr(self, ipstr):
        try:
            return IPy.IP(ipstr) in self.factory.cernet_nets_set
        except ValueError:
            pass

    def check_policy(self, identity):
        if not self.is_cernet_ipaddr(identity):
            self.discount_quota(self.discount)
        super(CERNetPolicy, self).check_policy(identity)


class CERNetPolicyFactory(PolicyFactory):

    policy_class = CERNetPolicy

    def __init__(self, discount):
        self.discount = discount
        cernet_resource = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data/cernet_ip.csv')
        cernet_nets = [r[0] for r in csv.reader(open(cernet_resource), delimiter='\t')]
        self.cernet_nets_set = IPy.IPSet([IPy.IP('%s/%s' % (ip_net, (ip_net.count('.') + 1) * 8), make_net=True) for ip_net in cernet_nets])
